ADB_COMMAND_SETTING_SHOW_TOUCHES = 'settings put system show_touches '
ADB_COMMAND_SETTING_POINTER_LOCATION = 'settings put system pointer_location '

ADB_COMMAND_GET_SOC_ARCHITECTURE = 'getprop ro.product.cpu.abi'



"""
    数字转汉字
    https://blog.csdn.net/PlusChang/article/details/72991191
    1~999999
"""

num = ['零', '一', '二', '三', '四', '五', '六', '七', '八', '九']
kin = ['十', '百', '千', '万', '零']


def del_surplus_0(x):
    if '零' in x:
        a = x.index('零')
        if a == 0:
            x.pop(0)
            del_surplus_0(x)
        else:
            if x[a + 2] in ['十', '百', '千', '万', '零']:
                if x[a + 1] != '万':
                    x.pop(a + 1)
                    del_surplus_0(x)
    return x


def del_surplus_unit(x):
    try:
        a = x.index('零')
        if x[a - 1] in ['十', '百', '千', '零']:
            x.pop(a - 1)
            del_surplus_unit(x[a + 1])
    except Exception:
        pass
    return x


def int_to_chinese(integer: int) -> str:
    # 数字转汉字列表
    integer = list(str(integer))
    for i in integer:
        integer[(integer.index(i))] = num[int(i)]
    # print(0, integer)

    integer.reverse()
    if len(integer) >= 2:
        integer.insert(1, kin[0])
        if len(integer) >= 4:
            integer.insert(3, kin[1])
            if len(integer) >= 6:
                integer.insert(5, kin[2])
                if len(integer) >= 8:
                    integer.insert(7, kin[3])
                    if len(integer) >= 10:
                        integer.insert(9, kin[0])
                        if len(integer) >= 12:
                            integer.insert(11, kin[1])
    # print(1, integer)

    if len(integer) >= 9:
        if integer[8] == '零':
            integer.pop(8)
    # print(2, integer)

    # 删除多余的0和单位
    integer = del_surplus_0(integer)
    # print(3, integer)

    # 删除多余的单位
    integer = del_surplus_unit(integer)
    # print(4, integer)

    # 删除末尾的零
    while len(integer) > 0 and integer[0] == '零':
        integer.pop(0)

    # 列表转字符串
    # print(50, integer)
    integer.reverse()
    integer = ''.join(integer)
    # print(51, integer)
    if integer == '':
        integer = '零'

    return integer




"""
    暂时重定向输出
    https://zhuanlan.zhihu.com/p/360137954
"""

import os
import sys


class HiddenPrints:
    def __init__(self, activated=True):
        self.activated = activated
        self.original_stdout = None

    def open(self):
        sys.stdout.close()
        sys.stdout = self.original_stdout

    def close(self):
        self.original_stdout = sys.stdout
        sys.stdout = open(os.devnull, 'w')

    def __enter__(self):
        if self.activated:
            self.close()

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.activated:
            self.open()



"""
    一系列adb操作，目前仅支持远程连接
"""
import os
from adb_shell.auth.keygen import keygen as generate_key
from adb_shell.adb_device import AdbDeviceTcp
from adb_shell.auth.sign_pythonrsa import PythonRSASigner
from .adb_command import *
# from .Settings import *
# from .Info import *
# from .Touch import *
# from .App import *
from importlib import import_module


def load_adbkey(adbkey_path, auto_create: bool = True):
    _dir = os.path.dirname(adbkey_path)
    if not os.path.exists(_dir):
        if not auto_create:
            raise FileNotFoundError(adbkey_path)
        os.makedirs(_dir)
    try:
        with open(adbkey_path, 'r') as f:
            key = f.read()
        with open(adbkey_path + '.pub', 'r') as f:
            pub = f.read()
        return [PythonRSASigner(pub, key)]
    except FileNotFoundError:
        if not auto_create:
            raise
        generate_key(adbkey_path)
        return load_adbkey(adbkey_path, auto_create=False)


class AyAdb:
    __connected = False
    __device: AdbDeviceTcp = None
    __adbkey = None

    def __init__(self, host: str, port: int, adbkey_path: str = './adbkey'):
        self.__adbkey = load_adbkey(adbkey_path)
        self.__device = AdbDeviceTcp(host, port, default_transport_timeout_s=9.)
        self.settings = import_module('.Settings', package='AyAdb').Settings(self)
        self.info = import_module('.Info', package='AyAdb').Info(self)
        self.touch = import_module('.Touch', package='AyAdb').Touch(self)
        self.app = import_module('.App', package='AyAdb').App(self)
        self.files = import_module('.Files', package='AyAdb').Files(self)
        self.images = import_module('.Images', package='AyAdb').Images(self)

    def __del__(self):
        if self.__connected:
            self.disconnect()

    def connect(self):
        if self.__connected:
            return True
        try:
            self.__connected = self.__device.connect(rsa_keys=self.__adbkey, auth_timeout_s=0.1)
        except Exception:
            raise Exception('连接失败')
        return self.__connected

    def disconnect(self):
        self.__connected = self.__device.close()

    def shell(self, cmd: str, exec_out: bool = False, decode: bool = True, timeout: float = None):
        if self.__connected:
            if exec_out:
                if timeout:
                    return self.__device.exec_out(cmd, decode=decode, transport_timeout_s=timeout, read_timeout_s=timeout)
                return self.__device.exec_out(cmd, decode=decode, transport_timeout_s=timeout)
            if timeout:
                return self.__device.shell(cmd, decode=decode, transport_timeout_s=timeout, read_timeout_s=timeout)
            return self.__device.shell(cmd, decode=decode, transport_timeout_s=timeout)
        return None

    def reboot(self, fastboot: bool = False):
        return self.__device.reboot(fastboot)

    def pull(self, src: str, dst: str):
        return self.__device.pull(src, dst)

    def push(self, src: str, dst: str):
        return self.__device.push(src, dst)

    def stat(self, path: str):
        return self.__device.stat(path)

    def list(self, path: str):
        return self.__device.list(path)



_PADCHAR = "="
# noinspection SpellCheckingInspection
_ALPHA = "LVoJPiCN2R8G90yg+hmFHuacZ1OWMnrsSTXkYpUq/3dlbfKwv6xztjI7DeBE45QA"


def _getbyte(s, i):
    x = ord(s[i])
    if x > 255:
        print("INVALID_CHARACTER_ERR: DOM Exception 5")
        exit(0)
    return x


def get_base64(s):
    # i = 0
    # b10 = 0
    x = []
    imax = len(s) - len(s) % 3
    if len(s) == 0:
        return s
    for i in range(0, imax, 3):
        b10 = (_getbyte(s, i) << 16) | (_getbyte(s, i + 1) << 8) | _getbyte(s, i + 2)
        x.append(_ALPHA[(b10 >> 18)])
        x.append(_ALPHA[((b10 >> 12) & 63)])
        x.append(_ALPHA[((b10 >> 6) & 63)])
        x.append(_ALPHA[(b10 & 63)])
    i = imax
    if len(s) - imax == 1:
        b10 = _getbyte(s, i) << 16
        x.append(_ALPHA[(b10 >> 18)] + _ALPHA[((b10 >> 12) & 63)] + _PADCHAR + _PADCHAR)
    else:
        b10 = (_getbyte(s, i) << 16) | (_getbyte(s, i + 1) << 8)
        x.append(_ALPHA[(b10 >> 18)] + _ALPHA[((b10 >> 12) & 63)] + _ALPHA[((b10 >> 6) & 63)] + _PADCHAR)
    return "".join(x)



"""
    自定义字符串类
    重写in支持判断列表或元组成员是否出现在字符串里
    重写find()支持查找子串列表或元组
"""
from typing import Union


class AyStr(str):
    def __contains__(self, item) -> bool:
        if isinstance(item, list):
            return len([member for member in item if member in self]) > 0
        if isinstance(item, tuple):
            return len([member for member in item if member in self]) > 0
        return super().__contains__(item)

    def find(self, __sub: Union[list, tuple, str], __start=None, __end=None) -> Union[list, tuple, int]:
        if isinstance(__sub, list):
            result = []
            for i in __sub:
                result.append(super().find(i, __start, __end))
            return result
        if isinstance(__sub, tuple):
            result = ()
            for i in __sub:
                result += (super().find(i, __start, __end),)
            return result
        return super().find(__sub, __start, __end)

    def startswith(self, __prefix: Union[list, tuple, str], __start=None, __end=None) -> bool:
        if isinstance(__prefix, list or tuple):
            for i in __prefix:
                if super().startswith(i, __start, __end):
                    return True
            return False
        return super().startswith(__prefix, __start, __end)

    def replace_all(self, __old: str, __new: str):
        tmp_str = str(self)
        while tmp_str.find(__old) != -1:
            tmp_str = tmp_str.replace(__old, __new)
        return AyStr(tmp_str)


if __name__ == '__main__':
    pass
    aa = AyStr('咱吃什么')
    word_what_self = ['我', '俺', '咱']
    print(aa.find(word_what_self))



"""
    自定义字典类
    键用半角实心点.做下级入口
    例 dict['web']['port'] == AyDict['web.port']
"""


class AyDict:
    __dict = {}

    def __init__(self, ori_dict=None):
        if ori_dict is None:
            ori_dict = {}
        self.__dict = ori_dict

    def __call__(self):
        return self.__dict

    def __eq__(self, other):
        return self.__dict == other.__dict

    def __contains__(self, ori_key: str):
        ori_key = ori_key.split('.')
        now_dict = self.__dict
        for key in ori_key:
            if key in now_dict:
                now_dict = now_dict[key]
            else:
                return False
        return True

    def __getitem__(self, ori_key: str):
        ori_key = ori_key.split('.')
        now_dict = self.__dict
        for key in ori_key:
            if key in now_dict:
                now_dict = now_dict[key]
            else:
                return None
        return now_dict

    def __setitem__(self, ori_key: str, ori_value):
        ori_key = ori_key.split('.')
        now_dict = self.__dict
        for key in ori_key[:-1]:
            if key not in now_dict:
                now_dict[key] = {}
            now_dict = now_dict[key]
        now_dict[ori_key[-1]] = ori_value
        return ori_value




if __name__ != '__main__':
    from .single_functions import *
    from .int_to_chinese import int_to_chinese
    from .AyDict import AyDict
    from .AyStr import AyStr
    from .get_file_format import get_file_format
    from .HiddenPrints import HiddenPrints


if __name__ == '__main__':
    print('This is a module, not a script.')



from AyAdb import AyAdb


class App:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    def get_main_activity(self, package_name):
        return self.__device.shell(f"cmd package resolve-activity -c android.intent.category.LAUNCHER {package_name} | sed -n '/name=/s/^.*name=//p'")

    def launch(self, package_name, activity=None, use_monkey=False):
        if use_monkey:
            return self.__device.shell(f'monkey -p {package_name} -c android.intent.category.LAUNCHER --pct-syskeys 0 1')
        return self.__device.shell(f'am start -n {package_name}/{activity if activity else self.get_main_activity(package_name)}')

    def stop(self, package_name):
        return self.__device.shell(f'am force-stop {package_name}')

    def install_apk(self, apk_path, replace=False, downgrade=False):
        filename = apk_path.split('/')[-1]
        self.__device.push(apk_path, '/data/local/tmp/' + filename)
        command = 'pm install '
        if replace:
            command += '-r '
        if downgrade:
            command += '-d '
        command += '/data/local/tmp/' + filename
        result = self.__device.shell(command)
        self.__device.shell('rm /data/local/tmp/' + filename)
        return result

    def uninstall_apk(self, package_name, keep_data=False):
        command = 'pm uninstall '
        if keep_data:
            command += '-k '
        command += package_name
        return self.__device.shell(command)

    def current_activity(self):
        tmp_str = self.__device.shell('dumpsys window | grep mCurrentFocus').strip()
        tmp_str = tmp_str.split(' ')[-1]
        tmp_str = tmp_str.split('/')
        return '/'.join(tmp_str[:-1]), tmp_str[-1][:-1]

    def get_installed_packages(self):
        tmp_list = []
        for line in self.__device.shell('pm list packages').split('\n'):
            try:
                tmp_list.append(line.split(':')[1].replace('\r', ''))
            except IndexError:
                pass
        return tmp_list




import base64
import html
import json
import random
from time import sleep
import requests as http
import websocket
import threading


class GocqConnection:
    __host = ''
    __access_token = ''
    __header = {
        'Authorization': 'Bearer '
    }
    __auto_connect = False

    __api_port = 0
    __api_url = ''
    Api = None

    __ws_port = 0
    __ws_url = ''
    __ws_connection = None
    __ws_connected = False
    __ws_thread_in_daemon = False
    __ws_thread = None
    __ws_event_message = None
    __ws_event_connected = None
    __ws_reconnect = False

    # QQ信息
    info = {
        'user_id': 0,
        'nick_name': '',
    }

    def is_connected(self):
        return self.__ws_connected

    # 根据配置生成url和header
    def __make_url(self):
        # self.__api_url = 'http://{}:{}'.format(self.__host, self.__api_port)
        self.Api = GocqApi(self.__host, self.__api_port, self.__access_token)
        self.__ws_url = 'ws://{}:{}'.format(self.__host, self.__ws_port)
        if not self.__access_token == '':
            self.__header['Authorization'] = 'Bearer ' + self.__access_token

    # 启动ws连接
    def __ws_connect(self):
        websocket.setdefaulttimeout(10)

        self.__ws_connection = websocket.WebSocketApp(
            url=self.__ws_url,
            header=self.__header,
            on_open=self._event_open,
            on_message=self._event_message,
            on_error=self._event_error,
            on_close=self._event_close,
            on_ping=self._event_ping,
            on_pong=self._event_pong,
            # on_cont_message=self._event_cont_message,
            # on_data=self._event_data,
        )

        self.__ws_thread = threading.Thread(
            target=self.__ws_connection.run_forever,
            daemon=self.__ws_thread_in_daemon,
            kwargs={
                'ping_interval': 60,
                'ping_timeout': 30,
                'ping_payload': '',
            }
        )
        self.__ws_thread.start()

    # 关闭ws连接
    def __ws_close(self):
        if self.__ws_thread.is_alive():
            self.__ws_connection.close()

    # 以下是ws事件方法
    def _event_open(self, _):
        # print(time.asctime(time.localtime(time.time())), 'open')
        self.__ws_connected = True

    def _event_message(self, _, message):
        message = json.loads(message)
        # 元事件
        if message['post_type'] == 'meta_event':
            # go-cqhttp生命周期事件
            if message['meta_event_type'] == 'lifecycle':
                # 连接成功
                self.info = self.Api.get_login_info()
                self.info['stat'] = self.Api.get_status()['stat']
                # print(self.info)
                if self.__ws_event_connected:
                    self.__ws_event_connected(self)
                return
            elif message['meta_event_type'] == 'heartbeat':
                # 心跳
                self.info['stat'] = message['status']['stat']
                return

        # print(message)
        # 其他消息
        # 没有配置消息回调，直接返回
        if not self.__ws_event_message:
            return
        result_dict = {
            # 'self_id': message['self_id'],
            'post_type': message['post_type'],
            'time': message['time'],
        }

        # 通知事件
        if message['post_type'] == 'notice':
            result_dict['notice_type'] = message['notice_type']
            # 群文件上传
            if message['notice_type'] == 'group_upload':
                # result_dict = message
                result_dict['message'] = ''
                result_dict['file'] = message['file']
                result_dict['sender'] = {}
                result_dict['sender']['user_id'] = message['user_id']
                result_dict['group_id'] = message['group_id']
            self.__ws_event_message(self, result_dict)
            return

        # 消息事件
        if message['post_type'] == 'message':
            message['message'] = html.unescape(message['message'])

            result_dict['message_type'] = message['message_type']
            result_dict['message'] = message['message']
            result_dict['message_id'] = message['message_id']

            result_dict['sender'] = {}
            result_dict['sender']['user_id'] = message['sender']['user_id']
            result_dict['sender']['nickname'] = message['sender']['nickname']

            # 群消息
            if message['message_type'] == 'group':
                result_dict['group_id'] = message['group_id']
                result_dict['sender']['card'] = message['sender']['card']
                result_dict['sender']['title'] = message['sender']['title']
                result_dict['sender']['role'] = message['sender']['role']
            elif message['message_type'] == 'private':
                pass

            self.__ws_event_message(self, result_dict)
            return

    def _event_error(self, ws, error):
        # print(time.asctime(time.localtime(time.time())), error)
        pass

    def _event_close(self, _, __, ___):
        # print(time.asctime(time.localtime(time.time())), 'close')
        self.__ws_connected = False
        if self.__ws_reconnect:
            self.__ws_connect()

    def _event_ping(self, _, message):
        # print(time.asctime(time.localtime(time.time())), 'got ping', message)
        pass

    def _event_pong(self, _, message: bytes):
        # print(time.asctime(time.localtime(time.time())), 'got pong')
        pass

    def __init__(self,
                 host: str,
                 ws_port: int,
                 api_port: int,
                 access_token: str,
                 auto_connect: bool = False,
                 daemon: bool = False,
                 on_message: callable = None,
                 on_connected: callable = None,
                 auto_reconnect: bool = False,
                 ):
        self.__host = host
        self.__ws_port = ws_port
        self.__api_port = api_port
        self.__access_token = access_token
        self.__make_url()

        self.__ws_reconnect = auto_reconnect
        self.__ws_thread_in_daemon = daemon

        self.__ws_event_message = on_message
        self.__ws_event_connected = on_connected

        self.__auto_connect = auto_connect
        if self.__auto_connect:
            self.start_connection()

    # 以下是公共方法
    # 启动ws连接
    def start_connection(self, in_daemon: bool = True):
        if in_daemon is not None:
            self.__ws_thread_in_daemon = in_daemon
        self.__ws_connect()

    # 关闭ws连接
    def stop_connection(self):
        self.__ws_close()


# noinspection HttpUrlsUsage
class GocqApi:
    __host = ''
    __port = 0
    __header = {
        'Authorization': 'Bearer '
    }
    __base_url = ''

    def __init__(self, host, port, access_token):
        self.__host = host
        self.__port = port
        self.__header['Authorization'] = 'Bearer ' + access_token
        self.__base_url = 'http://{}:{}'.format(self.__host, self.__port)

    def __go_api(self, api: str, method: int = 1, data: dict = None):
        if method == 0:
            result = http.get(self.__base_url + api, headers=self.__header, data=data)
        else:
            result = http.post(self.__base_url + api, headers=self.__header, data=data)
        return result.json()

    def send_private_message(self, send_to: int, message: str, auto_escape: bool = False):
        if send_to is None or message is None:
            return
        url = '/send_private_msg'
        data = {
            'user_id': send_to,
            'message': message,
            'auto_escape': auto_escape
        }
        result = self.__go_api(url, method=1, data=data)
        return result

    def send_group_message(self, send_to: int, message: str, auto_escape: bool = False):
        if send_to is None or message is None:
            return
        url = '/send_group_msg'
        data = {
            'group_id': send_to,
            'message': message,
            'auto_escape': auto_escape
        }
        # sleep(random.uniform(0.1, 0.9))
        result = self.__go_api(url, method=1, data=data)
        return result

    def send_message(self, msg: dict, auto_escape: bool = False):
        if 'group_id' in msg.keys():
            return self.send_group_message(msg['group_id'], msg['message'], auto_escape)
        else:
            return self.send_private_message(msg['sender']['user_id'], msg['message'], auto_escape)

    def get_login_info(self):
        url = '/get_login_info'
        result = self.__go_api(url)
        return result['data']

    def get_status(self):
        url = '/get_status'
        result = self.__go_api(url)
        # print(result)
        return result['data']


class CqCode:
    @staticmethod
    def face(face_id: int):
        return '[CQ:face,id={}]'.format(face_id)

    @staticmethod
    def image(file, image_type: str = 'normal', use_cache: bool = 1, show_id: int = 40000, threads: int = 2):
        code = '[CQ:image,file='
        if isinstance(file, str):
            code += f'{file}'
        elif isinstance(file, bytes):
            code += f'base64://{base64.b64encode(file).decode()}'
        if image_type == 'flash':
            code += ',type=flash'
        elif image_type == 'show':
            code += ',type=show'
            code += f',id={show_id}'
        if use_cache == 0:
            code += ',cache=0'
        code += f',c={threads}]'
        return code

    @staticmethod
    def image_local(file_path: str, image_type: str = 'normal',
                    use_cache: bool = 1, show_id: int = 40000, threads: int = 2):
        try:
            with open(file_path, 'rb') as f:
                return CqCode.image(f.read(), image_type, use_cache, show_id, threads)
        except FileExistsError:
            return CqCode.image('file:///' + file_path, image_type, use_cache, show_id, threads)

    # [CQ:at,qq=10001000]
    @staticmethod
    def at(user_id):
        return f'[CQ:at,qq={user_id}]'

    # [CQ:record,file=http://baidu.com/1.mp3]
    @staticmethod
    def record(file, use_cache: bool = 1):
        code = '[CQ:record,file='
        if isinstance(file, str):
            code += f'{file}'
        elif isinstance(file, bytes):
            code += f'base64://{base64.b64encode(file).decode()}'
        if use_cache == 0:
            code += ',cache=0'
        code += ']'
        return code

    @staticmethod
    def record_local(file_path: str, use_cache: bool = 1):
        try:
            with open(file_path, 'rb') as f:
                return CqCode.record(f.read(), use_cache)
        except FileExistsError:
            return CqCode.record('file:///' + file_path, use_cache)


from AyAdb import AyAdb


class Touch:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    def tap(self, x, y):
        return self.__device.shell(f'input tap {x} {y}', True)

    def swipe(self, x1, y1, x2, y2, duration=None):
        if duration is None:
            return self.__device.shell(f'input swipe {x1} {y1} {x2} {y2}', True)
        else:
            return self.__device.shell(f'input swipe {x1} {y1} {x2} {y2} {duration}', True)



from AyAdb import AyAdb
import re


class Info:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    # Application Binary Interface
    # like: arm64-v8a
    def cpu_abi(self):
        return self.__device.shell('getprop ro.product.cpu.abi').strip()

    # like: 7.1.2
    def version_release(self):
        return self.__device.shell('getprop ro.build.version.release').strip()

    # like: 25
    def version_sdk(self):
        return int(self.__device.shell('getprop ro.build.version.sdk').strip())

    # like: (720, 1280)
    def screen_resolution(self):
        size_str = self.__device.shell('wm size').strip()
        re_result = re.search(r'Override size: (\d+)x(\d+)', size_str)
        if re_result:
            return int(re_result.group(1)), int(re_result.group(2))
        else:
            re_result = re.search(r'Physical size: (\d+)x(\d+)', size_str)
            if re_result:
                return int(re_result.group(1)), int(re_result.group(2))
            else:
                return 0, 0

    # like: 320
    def screen_density(self):
        dpi_str = self.__device.shell('wm density').strip()
        dpi_str = dpi_str.split('Physical density: ')[1]
        dpi_str = dpi_str.split(' ')[0]
        return int(dpi_str)



from AyAdb import AyAdb


class Files:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    def create_directory(self, directory):
        self.__device.shell('mkdir -p ' + directory)

    def is_exists(self, file):
        return self.__device.shell('[ -e ' + file + ' ] && echo 1 || echo 0').strip() == '1'

    def is_directory(self, directory):
        return self.__device.shell('[ -d ' + directory + ' ] && echo 1 || echo 0').strip() == '1'

    def is_file(self, file):
        return self.__device.shell('[ -f ' + file + ' ] && echo 1 || echo 0').strip() == '1'

    def del_file(self, file):
        self.__device.shell('rm -rf ' + file)

    def del_directory(self, directory):
        self.del_file(directory)



# 获取自身运行时长
def get_self_uptime():
    import os
    from datetime import datetime
    import psutil
    start_time = psutil.Process(os.getpid()).create_time()
    curr_time = datetime.now()
    return curr_time - start_time


# 获取开机时长
def get_machine_uptime():
    from datetime import datetime
    import psutil
    boot_time = datetime.fromtimestamp(psutil.boot_time())
    curr_time = datetime.now()
    return curr_time - boot_time


# 获取自身ip
# https://www.zhihu.com/question/49036683/answer/1243217025
def get_self_ip():
    s = None
    try:
        import socket
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        return s.getsockname()[0]
    finally:
        if s:
            s.close()


# 汉字转数字
# https://www.iplaypy.com/code/base/b2600.html
def chinese_to_int(_cn: str) -> int:
    CN_NUM = {
        '〇': 0,
        '一': 1,
        '二': 2,
        '三': 3,
        '四': 4,
        '五': 5,
        '六': 6,
        '七': 7,
        '八': 8,
        '九': 9,

        '零': 0,
        '壹': 1,
        '贰': 2,
        '叁': 3,
        '肆': 4,
        '伍': 5,
        '陆': 6,
        '柒': 7,
        '捌': 8,
        '玖': 9,

        '貮': 2,
        '两': 2,
    }
    CN_UNIT = {
        '十': 10,
        '拾': 10,
        '百': 100,
        '佰': 100,
        '千': 1000,
        '仟': 1000,
        '万': 10000,
        '萬': 10000,
        '亿': 100000000,
        '億': 100000000,
        '兆': 1000000000000,
    }
    lcn = list(_cn)
    unit = 0  # 当前的单位
    ldig = []  # 临时数组

    while lcn:
        cndig = lcn.pop()

        if cndig in CN_UNIT.keys():
            unit = CN_UNIT.get(cndig)
            if unit == 10000:
                ldig.append('w')  # 标示万位
                unit = 1
            elif unit == 100000000:
                ldig.append('y')  # 标示亿位
                unit = 1
            elif unit == 1000000000000:  # 标示兆位
                ldig.append('z')
                unit = 1

            continue

        else:
            dig = CN_NUM.get(cndig)

            if unit:
                dig = dig * unit
                unit = 0

            ldig.append(dig)

    if unit == 10:  # 处理10-19的数字
        ldig.append(10)

    ret = 0
    tmp = 0

    while ldig:
        x = ldig.pop()

        if x == 'w':
            tmp *= 10000
            ret += tmp
            tmp = 0

        elif x == 'y':
            tmp *= 100000000
            ret += tmp
            tmp = 0

        elif x == 'z':
            tmp *= 1000000000000
            ret += tmp
            tmp = 0

        else:
            tmp += x

    ret += tmp
    return ret


# 也是汉字转数字
# https://www.jb51.net/article/114789.htm
def chinese_to_int_e(_cn: str) -> int:
    common_used_numerals_tmp = {
        '零': 0,
        '一': 1,
        '二': 2,
        '两': 2,
        '三': 3,
        '四': 4,
        '五': 5,
        '六': 6,
        '七': 7,
        '八': 8,
        '九': 9,
        '十': 10,
        '百': 100,
        '千': 1000,
        '万': 10000,
        '亿': 100000000
    }
    common_used_numerals = common_used_numerals_tmp
    total = 0
    r = 1  # 表示单位：个十百千...
    for i in range(len(_cn) - 1, -1, -1):
        val = common_used_numerals.get(_cn[i])
        if val >= 10 and i == 0:  # 应对 十三 十四 十*之类
            if val > r:
                r = val
                total = total + val
            else:
                r = r * val
                # total =total + r * x
        elif val >= 10:
            if val > r:
                r = val
            else:
                r = r * val
        else:
            total = total + r * val
    return total




import os
from random import random

from AyAdb import AyAdb


class Images:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    def capture_screen(self, raw=False, use_file_system=False) -> bytes:
        # return PNG bytes if not raw
        command = 'screencap'
        if not raw:
            command += ' -p'
        if not use_file_system:
            return self.__device.shell(command, exec_out=True, decode=False, timeout=60)
        else:
            rand_name = str(int(random() * 100))
            command += '  /data/local/tmp/screenshot' + rand_name
            self.__device.shell(command, exec_out=True)
            self.__device.pull('/data/local/tmp/screenshot' + rand_name, 'screenshot' + rand_name)
            self.__device.files.del_file('/data/local/tmp/screenshot' + rand_name)
            with open('screenshot' + rand_name, 'rb') as f:
                file_content = f.read()
            os.remove('screenshot' + rand_name)
            return file_content

        # HOW TO USE RAW MODE
        # https://stackoverflow.com/questions/22034959/what-format-does-adb-screencap-sdcard-screenshot-raw-produce-without-p-f
        # https://stackoverflow.com/questions/7142169/pils-image-frombuffer-expected-data-length-when-using-ctypes-array
        # screenshot = device.images.capture_screen(raw=False)[12:]
        # screen_width, screen_height = device.info.screen_resolution()
        # # from ctypes import c_ubyte, cast, POINTER
        # # tmp = cast(screenshot, POINTER(c_ubyte*4*screen_width*screen_height)).contents
        # imm = Image.frombuffer('RGBA', (screen_width, screen_height), screenshot, 'raw', 'RGBX', 0, 1)
        # imm.show()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='KenkoAdvance',
    version='1.0.0',
    author='AkagiYui',
    author_email='akagiyui@yeah.net',
    description='自用的一些功能',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/AkagiYui/KenkoAdvance',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(where='src'),
    package_dir={"": "src"},
    python_requires=">=3.9",
    license='MIT',
)



"""
    文本转Midi
    作者: Scarlett凛子
    QQ: 1587524

    格式：多行文本，包括参数行与音符行。行内出现参数时则视为参数行，不解析音符。

    参数行：
        调号：支持大调(1=)和小调(6=)记号，另支持低八度(末尾带2)或高八度(末尾带4)。
            示例：1=C，6=A，1=Bb，1=C2，1=C4。默认1=C。
        节拍：数字+bpm。
            示例：140bpm。默认120。

    音符行：01234567是八个音符，每个音符后面加符号来表示时值和音高。
        音高：#:升半音，b:降半音，.:高八度，,:低八度。
        时值：*:0.5拍，-:1拍，~:1.5拍，=:2拍，+:4拍，^:0.25拍(16分)，':0.125拍(32分)，默认0.5拍。

    文本示例
    6=B 138bpm
    3571.753-2121253-
    211-6,1432121253-
    3571.753-2121253-
    211-6-5-357771.2.7577=~66+
"""
import re
import mido

from KenkoAdvance import AyStr

PITCH_NAME = {'A': -3, 'B': -1, 'C': 0, 'D': 2, 'E': 4, 'F': 5, 'G': 7}
PITCH_LIST = [0, 2, 4, 5, 7, 9, 11]

BASIC_PITCH_ADD = [(0, '6', 3), (2, '#', 1), (2, 'b', -1), (3, '2', -12), (3, '4', 12)]
PITCH_ADD = {'#': 1, 'b': -1, '.': 12, ',': -12}
TIME_ADD = {'*': 0.5, '-': 1, '~': 1.5, '=': 2, '+': 4, '^': 0.25, "'": 0.125}


class TextToMidi:
    text = ''
    bpm = 120
    basic_pitch = 60
    score_list = []
    midi_file = None

    def __init__(self, _text: str = ''):
        self.set_text(_text)

    def set_text(self, _text: str):
        self.text = _text

    def decode(self):
        _basic_pitch = 60
        _bpm = 120
        lines = self.text.split('\n')
        _main_list = []
        for line in lines:
            is_param_line = False

            key_signature = re.search('([16])=([ABCDEFG])([#b]?)([24]?)', line)
            if key_signature is not None:
                is_param_line = True
                basic_pitch = 60
                key_signature = key_signature.groups()
                basic_pitch += PITCH_NAME[key_signature[1]]
                for x in BASIC_PITCH_ADD:
                    if key_signature[x[0]] == x[1]:
                        basic_pitch += x[2]
                _basic_pitch = basic_pitch

            meter = re.search('(\d*)bpm', line.lower())
            if meter is not None:
                is_param_line = True
                _bpm = eval(meter.groups()[0])

            if is_param_line:
                continue

            score = AyStr(line).replace_all(' ', '') + '0'
            little_list = []
            pitch = -1
            length = -1
            for char in score:
                if char in '01234567':
                    if length == 0:
                        length = 0.5
                    little_list.append((_basic_pitch + pitch, length, _bpm))

                    if char == '0':
                        pitch = -1000
                    else:
                        pitch = PITCH_LIST[eval(char) - 1]
                    length = 0
                    continue

                try:
                    pitch += PITCH_ADD[char]
                except KeyError:
                    pass

                try:
                    length += TIME_ADD[char]
                except KeyError:
                    pass
            little_list.pop(0)
            _main_list += little_list
        self.score_list = _main_list

    def make(self, velocity: int = 63):
        self.decode()
        self.midi_file = mido.MidiFile()  # 新建midi文件
        track = mido.MidiTrack()  # 新建midi轨道
        self.midi_file.tracks.append(track)  # 加入这条轨道
        track.append(mido.Message('program_change', channel=0, program=0, time=0))
        for note in self.score_list:
            if note[0] < 0:
                track.append(mido.Message('note_on', note=0, velocity=0, time=0))
                track.append(mido.Message('note_off',
                                          note=0,
                                          velocity=0,
                                          time=int(note[1] * (60 / note[2]) * 1000)))
            else:
                track.append(mido.Message('note_on', note=note[0], velocity=velocity, time=0))
                track.append(mido.Message('note_off',
                                          note=note[0],
                                          velocity=velocity,
                                          time=int(note[1] * (60 / note[2]) * 1000)))

    def save(self, file_path: str):
        self.midi_file.save(file_path)

    def get_length(self):
        if not self.midi_file:
            return None
        return self.midi_file.length


if __name__ == '__main__':
    pass
    text = '''1=Eb 130bpm
0-356*^4^*3212656^32*^1-
6,12^*7,^*433.2.71.*^7^*565
356*^4^*327655671.-32-1==
03,3,03,05,^6,^1^5^6^53^*6,123^6,^1.^7^5^35^*65326,3^21^*6,5,#437,21^5^*435,5,^6,^1^5^3^21^*6,123^6,^1.^7^5^35^*6532^6,^1^5^3^21^*6,5,#437,05,^6,^1^2^3^6^5^2^3^6,^1'''
    # text = '70bpm\n1155665-4433221-5544332-5544332-1155665-4433221-'
    aa = TextToMidi(text)
    aa.make(velocity=127)
    print('Music Length: %.2f seconds' % aa.get_length())
    aa.save(r'D:\OneDrive\Projects\GocqTools\temp\new2.mid')



from AyAdb import AyAdb


class Settings:
    __device: AyAdb = None

    def __init__(self, device=None):
        self.__device = device

    def show_touches(self, value=None):
        if value is None:
            return self.__device.shell('settings get system show_touches').strip() == '1'
        return self.__device.shell(f'settings put system show_touches {("1" if value else "0")}')

    def pointer_location(self, value=None):
        if value is None:
            return self.__device.shell('settings get system pointer_location').strip() == '1'
        return self.__device.shell(f'settings put system pointer_location {("1" if value else "0")}')

